<?php
/**
 * Init wwcAmazonSyncronize
 * http://www.aa-team.com
 * =======================
 *
 * @author		Andrei Dinca, AA-Team
 * @version		0.1
 */
if (class_exists('wwcAmazonSyncronize') != true) {
class wwcAmazonSyncronize
{
	/**
	 * cfg
	 *
	 * @var array
	 */
	private $cfg = array(
		'available_setup' => array('hourly', 'twicedaily', 'daily')
	);


	/**
	 * wwcAmzAff
	 *
	 * @var array
	 */
	private $wwcAmzAff = null;

	/**
	 * wwcAmzAff
	 *
	 * @var db
	 */
	private $db = null;


	/**
	 * @params main class object
	 */
	public function __construct($wwcAmzAff)
	{
		global $wpdb;
		$this->db = $wpdb;
		$this->wwcAmzAff = $wwcAmzAff;
		$this->cfg['available_setup'] = unserialize(get_option('wwcAmzAff_sync'));
	}

	public function updateTheTask()
	{
		$recc = $this->cfg['available_setup']['recurrence'];
		$start_hour = $this->cfg['available_setup']['start'];
  
		// check if is valid
		//if( !in_array($recc, $this->cfg['available_setup']) ) return false;

		// now remove the current task from queue
		$this->removeCurrentTask();

		// create timestamp from hour
		$str_date = date('y-m-d');
		$timestamp = strtotime( $str_date );
		$timestamp = $timestamp + ($start_hour * 60);

		// add new task
		wp_schedule_event( $timestamp, $recc, 'wwcAmzAff_SyncProducts_event');

		die('OK');
	}

	private function removeCurrentTask()
	{
		wp_clear_scheduled_hook( 'wwcAmzAff_SyncProducts_event' );
	}

	// store into cfg array, no returns
	final private function updateSyncRules()
	{
		$this->cfg['sync_rules'] = array(
			'title' 		=> isset($this->cfg['available_setup']['title']) ? true : false,
			'reviews' 		=> isset($this->cfg['available_setup']['reviews']) ? true : false,
			'price' 		=> isset($this->cfg['available_setup']['price']) ? true : false,
			'url' 			=> isset($this->cfg['available_setup']['url']) ? true : false,
			'content' 		=> isset($this->cfg['available_setup']['desc']) ? true : false,
			'sku' 			=> isset($this->cfg['available_setup']['sku']) ? true : false,
			'image' 		=> isset($this->cfg['available_setup']['image']) ? true : false
		);
	}
	
	public function updateTheProduct( $asins=array(), $return='die' )
	{
		if ( empty($asins) ) {
			if( $return == 'print_return' ){
				echo "No ASINs provided!";
			}else{
				die(json_encode(array(
					'status' => 'invalid',
					'msg' => "No ASINs provided!"
				)));
			}			
		}

		if ( !is_array($asins) ) {
			global $wpdb;
			
			$post_id = $wpdb->get_var("select a.post_id from $wpdb->postmeta as a where 1=1 and a.meta_key = '_amzASIN' and a.meta_value='" . ($asins) . "';");
			
			$asins = array($post_id => $asins);
		}
 
		$this->updateSyncRules();

		// load the amazon webservices client class
		$amazon_settings = $this->wwcAmzAff->getAllSettings('array', 'amazon');
		$plugin_uri = $this->wwcAmzAff->cfg['paths']['plugin_dir_url'] . 'modules/amazon/';
		require_once( $this->wwcAmzAff->cfg['paths']['plugin_dir_path'] . '/lib/scripts/amazon/aaAmazonWS.class.php');
		
		// create new amazon instance
		$aaAmazonWS = new aaAmazonWS(
			$amazon_settings['AccessKeyID'],
			$amazon_settings['SecretAccessKey'],
			$amazon_settings['country'],
			$this->wwcAmzAff->main_aff_id()
		);

		$asins_updated = array();
		// create request by ASIN
		// Large,ItemAttributes,Offers
		$products = $aaAmazonWS->responseGroup('Large,ItemAttributes,OfferFull,Variations,Reviews,PromotionSummary')->optionalParameters(array('MerchantId' => 'All'))->lookup( implode(",", $asins) );
  
		if($products['Items']["Request"]["IsValid"] == "True"){
			$arrProds = array();
  			if ( isset($products['Items']['Request']['ItemLookupRequest']['ItemId'])
				&& !is_array($products['Items']['Request']['ItemLookupRequest']['ItemId'])
				|| ( is_array($products['Items']['Request']['ItemLookupRequest']['ItemId']) && count($products['Items']['Request']['ItemLookupRequest']['ItemId']) <= 1 ) ) {
  				$arrProds[] = $products['Items']['Item'];
  			} else {
  				$arrProds = $products['Items']['Item'];  				
  			}

			foreach ($arrProds as $thisProd) {
 
				$localID = 0;
				if(count($thisProd) > 0) {
					// start creating return array
					$retProd = array();
	
					$retProd['ASIN'] = $thisProd['ASIN'];
	
					// CustomerReviews url
					if($thisProd['CustomerReviews']['HasReviews']){
						$retProd['CustomerReviewsURL'] = $thisProd['CustomerReviews']['IFrameURL'];
					}
	
					// DetailPageURL
					$retProd['DetailPageURL'] = $thisProd['DetailPageURL'];
	
					// product title
					$retProd['Title'] = isset($thisProd['ItemAttributes']['Title']) ? $thisProd['ItemAttributes']['Title'] : '';
	
					// Binding
					$retProd['Binding'] = isset($thisProd['ItemAttributes']['Binding']) ? $thisProd['ItemAttributes']['Binding'] : '';
	
					// ProductGroup
					$retProd['ProductGroup'] = isset($thisProd['ItemAttributes']['ProductGroup']) ? $thisProd['ItemAttributes']['ProductGroup'] : '';
	
					// SKU
					$retProd['SKU'] = isset($thisProd['ItemAttributes']['SKU']) ? $thisProd['ItemAttributes']['SKU'] : '';
	
					// Feature
					$retProd['Feature'] = isset($thisProd['ItemAttributes']['Feature']) ? $thisProd['ItemAttributes']['Feature'] : '';
					
					// EditorialReviews
					$retProd['EditorialReviews'] = isset($thisProd['EditorialReviews']['EditorialReview']['Content']) ? $thisProd['EditorialReviews']['EditorialReview']['Content'] : '';
					
					// The product Offers
					$retProd['Offers'] = isset($thisProd['Offers']) ? $thisProd['Offers'] : array();
					$retProd['OfferSummary'] = isset($thisProd['OfferSummary']) ? $thisProd['OfferSummary'] : array();
					
					// The product Item attribues
					$retProd['ItemAttributes'] = isset($thisProd['ItemAttributes']) ? $thisProd['ItemAttributes'] : array();
					
					// The product VariationSummary
					$retProd['VariationSummary'] = isset($thisProd['VariationSummary']) ? $thisProd['VariationSummary'] : array();


					$requestData = array();
					$requestData['debug_level'] = isset($_REQUEST['debug_level']) ? (int)$_REQUEST['debug_level'] : 0;
					// print some debug if requested
					if( $requestData['debug_level'] > 0 ) {
						if( $requestData['debug_level'] == 1) var_dump('<pre>', $retProd,'</pre>');
						if( $requestData['debug_level'] == 2) var_dump('<pre>', $product ,'</pre>');

						die;
					}
  
					foreach ($asins as $code_key => $code_value) {
						if( $thisProd['ASIN'] == $code_value){
							$localID = $code_key;
							$asins_updated[$localID] = $thisProd['ASIN'];
						}
					}
  
					$this->wwcAmzAff->updateWooProduct($retProd, $this->cfg['sync_rules'], $localID);
	
					$product_meta['product'] = array();
					$product_meta['product']['price_update_date'] = get_post_meta($localID, "_price_update_date", true);
					$product_meta['product']['sales_price'] = get_post_meta($localID, "_sale_price", true);
					$product_meta['product']['regular_price'] = get_post_meta($localID, "_regular_price", true);
					$product_meta['product']['price'] = get_post_meta($localID, "_price", true);
					
					if ( empty($product_meta['product']['sales_price']) && empty($product_meta['product']['regular_price']) ) {
						$product_meta['product']['variation_price'] = array('min' => get_post_meta($localID, "_min_variation_price", true), 'max' => get_post_meta($localID, "_max_variation_price", true));
					}
  
					// update the last sync date
					$last_sync_date = date('F j, Y, g:i a');
					update_post_meta( $localID, "_last_sync_date", $last_sync_date );
					
					// update wwcAmzAff_remaining_at, take this as marker for features sync
					update_option('wwcAmzAff_last_updated_product', $localID);
					
					if( $return == 'print_return' ){
						echo 'Update, OK - ' . ( $localID ) . ' <br />';
					}else{
						die(json_encode(array(
							'status' 	=> 'valid',
							'data' 		=> array(
								'regular_price' => (isset($product_meta['product']['regular_price']) && (float)$product_meta['product']['regular_price'] > 0 ? woocommerce_price( $product_meta['product']['regular_price'] ) : '&#8211;'),
								'sales_price' => (isset($product_meta['product']['sales_price']) && (float)$product_meta['product']['sales_price'] > 0 ? woocommerce_price( $product_meta['product']['sales_price'] ) : '&#8211;'),
								'last_sync_date' => $last_sync_date,
								'variation_price' => array(
									'min' => (isset($product_meta['product']['variation_price']['min']) && (float)$product_meta['product']['variation_price']['min'] > 0 ? woocommerce_price( $product_meta['product']['variation_price']['min'] ) : '&#8211;'),
									'max' => (isset($product_meta['product']['variation_price']['max']) && (float)$product_meta['product']['variation_price']['max'] > 0 ? woocommerce_price( $product_meta['product']['variation_price']['max'] ) : '&#8211;')
								)
							)
						)));
					}
					
					if( (int)$this->cfg['available_setup']['sleep'] > 0 ) sleep( (int)$this->cfg['available_setup']['sleep'] );

				} // end if(count($thisProd) > 0){
				/*else{
					if( $return == 'print_return' ){
						// delete the product if no longer available on Amazon
						wp_trash_post( $localID );
	
						echo 'Invalid update for: ' . ( $localID ) . '. Error: <i>' . ( $products['Items']['Request']['Errors']['Error']['Message'] ) . '</i> <br />';
					}else{
						die( json_encode( array(
							'status' 	=> 'invalid',
							'msg'		=> $products['Items']['Request']['Errors']['Error']['Message']
						) ) );
					}
				}*/
			} // end foreach ($products['Items']['Item'] as $thisProd) {

			// remove ASINs not found on Amazon anymore!
			$asins_notfound = array_diff($asins, $asins_updated);
			if ( count($asins_notfound) > 0 ) {
				
			$__sync_prod_notfound = get_option('wwcAmzAff_sync_prod_notfound', true);
			if ( !is_array($__sync_prod_notfound) || empty($__sync_prod_notfound) )
				$__sync_prod_notfound = array();
			$__sync_prod_notfound = $__sync_prod_notfound + $asins_notfound;
			$__sync_prod_notfound = array_unique($__sync_prod_notfound);
			update_option('wwcAmzAff_sync_prod_notfound', $__sync_prod_notfound);
			
			foreach ($asins_notfound as $localID => $asin) {
					if( $return == 'print_return' ){
						// delete the product if no longer available on Amazon
						wp_trash_post( $localID );
	
						echo 'Invalid update for: (ID: ' . ( $localID ) . ', ASIN: ' . ( $asin ) . '). Error: <i>' . ( isset($products['Items']['Request']['Errors']['Error']['Message']) ? $products['Items']['Request']['Errors']['Error']['Message'] : serialize($products['Items']['Request']['Errors']['Error']) ) . '</i> <br />';
					}else{
						die( json_encode( array(
							'status' 	=> 'invalid',
							'msg'		=> isset($products['Items']['Request']['Errors']['Error']['Message']) ? $products['Items']['Request']['Errors']['Error']['Message'] : serialize($products['Items']['Request']['Errors']['Error'])
						) ) );
					}
			} // end foreach ($asins_notfound as $localID => $asin) {
			}

		}
		else{
			if( $return == 'print_return' ){
				echo "Can't get product by given ASIN: " . $asin;
			}else{
				die(json_encode(array(
					'status' => 'invalid',
					'msg' => "Can't get product by given ASIN: " . $asin
				)));
			}
		}
	}

	public function updateTheProducts()
	{
		$this->updateSyncRules(); 
  
		$metaPrefix = '_';
		$__products = $this->getAllPublishProducts();
		//var_dump('<pre>', $__products, '</pre>'); die('debug...');  
		$products = $this->extract_products_by_request_size( $__products );
		//var_dump('<pre>', $products, '</pre>'); die('debug...'); 
  
		if(count($products) > 0) {
			$amz_products = array(); 
			foreach ($products as $key => $value){
				$asin = get_post_meta( $value['ID'], '_amzASIN', true );
				$amz_products[$value['ID']] = $asin;
			}
			$amz_products = array_unique($amz_products);
			foreach (array_chunk($amz_products, 10, true) as $products) {
				$this->updateTheProduct( $products, 'print_return' );
			}  
		}else{
			die('No products selected, all products are updated!');
		}
	}
	
	private function extract_products_by_request_size( $products=array() )
	{
		if(count($products) > 0 ){
  
			// Sync all products on same request
			if( $this->cfg['available_setup']['sync_products_per_request'] == 0 ){
				return $products;
			}

			// range size 
			//$nrOfProducts = (int) $this->cfg['available_setup']['sync_products_per_request'] * 10;
			$nrOfProducts = (int) $this->cfg['available_setup']['sync_products_per_request'];
			if( $nrOfProducts == 0 ) $nrOfProducts = 10;
  
			$last_updated_product = get_option('wwcAmzAff_last_updated_product', true);

			$selectedProducts = array();
			$added = 0;
			$start_from_here = false;
			foreach ($products as $key => $value) {
				
				if( $start_from_here == true && $added < $nrOfProducts ){
					$selectedProducts[] = $value;
					$added++;
				}
				
				if( $value['ID'] == $last_updated_product ){
					$start_from_here = true;
				}
			}
			
			// if the searched products was to the finish and don't take enough products add first X products
			if( count($selectedProducts) < $nrOfProducts ){
				$selectedProducts = array_merge( $selectedProducts, array_slice($products, 0, ($nrOfProducts - count($selectedProducts)) ) );  
			}
			
			return $selectedProducts;
		}
	}
	
	final protected function getAllPublishProducts()
	{
	
		$ret = array();
		$args = array();
		$args['post_type'] = 'product';

		$args['meta_key'] = '_amzASIN';
		$args['meta_value'] = '';
		$args['meta_compare'] = '!=';

		// show all posts
		$args['fields'] = 'ids';
		$args['posts_per_page'] = '-1';
		
		$loop = new WP_Query( $args );
		$cc = 0;
		$html = array(); 
		while ( $loop->have_posts() ) : $loop->the_post();
			global $post;
		
			// add product only if not been updated in recurrence time
			$last_sync_date = get_post_meta($post, '_last_sync_date', true);
			
			//if ( 1 ) {
			if ( ( time() >= (strtotime($last_sync_date) + (3600 * $this->cfg['available_setup']["recurrence"])) )
				|| trim($last_sync_date) == "" ) {
                $ret[] = array( 'ID' => $post );
            }
			
		endwhile;
		return $ret;
	}
}
}

add_action('wp_ajax_wwcAmzAffSyncUpdate', 'wwcAmzAffSyncUpdate');
function wwcAmzAffSyncUpdate() {
	global $wwcAmzAff;

	$sync = new wwcAmazonSyncronize($wwcAmzAff);
	$sync->updateTheTask();
}

//add_action('wwcAmzAff_SyncProducts_event', 'wwcAmzAff_SyncProducts_event');
function wwcAmzAff_SyncProducts_event() {
	global $wwcAmzAff;

	$sync = new wwcAmazonSyncronize($wwcAmzAff);
	$sync->updateTheProducts();
}

require_once( 'tail.php' );