<?php
/*
* Define class wwcAmzAffTailSyncMonitor
* Make sure you skip down to the end of this file, as there are a few
* lines of code that are very important.
*/
!defined('ABSPATH') and exit;
if (class_exists('wwcAmzAffTailSyncMonitor') != true) {
    class wwcAmzAffTailSyncMonitor
    {
        /*
        * Some required plugin information
        */
        const VERSION = '1.0';

        /*
        * Store some helpers config
        */
		public $the_plugin = null;

		private $module_folder = '';
		private $module = '';

		static protected $_instance;

        /*
        * Required __construct() function that initalizes the AA-Team Framework
        */
        public function __construct()
        {
        	global $wwcAmzAff;

        	$this->the_plugin = $wwcAmzAff;
			$this->module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . 'modules/synchronization/';
			$this->module = $this->the_plugin->cfg['modules']['synchronization'];

			if (is_admin()) {
	            add_action('admin_menu', array( &$this, 'adminMenu' ));
			}

			// ajax  helper
			add_action('wp_ajax_wwcAmzAffSyncProduct', array( &$this, 'ajax_request' ));
        }

		/**
	    * Singleton pattern
	    *
	    * @return wwcAmzAffTailSyncMonitor Singleton instance
	    */
	    static public function getInstance()
	    {
	        if (!self::$_instance) {
	            self::$_instance = new self;
	        }

	        return self::$_instance;
	    }

		/**
	    * Hooks
	    */
	    static public function adminMenu()
	    {
	       self::getInstance()
	    		->_registerAdminPages();
	    }

	    /**
	    * Register plug-in module admin pages and menus
	    */
		protected function _registerAdminPages()
    	{ 
    		add_submenu_page(
    			$this->the_plugin->alias,
    			$this->the_plugin->alias . " " . __('Synchronization log', $this->the_plugin->localizationName),
	            __('Synchronization log'),
	            'manage_options',
	            $this->the_plugin->alias . "_synclog",
	            array($this, 'display_index_page')
	        );

			return $this;
		}

		public function display_index_page()
		{
			$this->printBaseInterface();
		}
		
		/*
		* printBaseInterface, method
		* --------------------------
		*
		* this will add the base DOM code for you options interface
		*/
		private function printBaseInterface()
		{
			global $wpdb;
?>
		<script type="text/javascript" src="<?php echo $this->module_folder;?>app.class.js" ></script>
		<div id="wwcAmzAff-wrapper" class="fluid wrapper-wwcAmzAff">
			
			<?php
			// show the top menu
			wwcAmzAffAdminMenu::getInstance()->make_active('info|synchronization_log')->show_menu(); 
			?>
			
			<!-- Main loading box -->
			<div id="wwcAmzAff-main-loading">
				<div id="wwcAmzAff-loading-overlay"></div>
				<div id="wwcAmzAff-loading-box">
					<div class="wwcAmzAff-loading-text"><?php _e('Loading', $this->the_plugin->localizationName);?></div>
					<div class="wwcAmzAff-meter wwcAmzAff-animate" style="width:86%; margin: 34px 0px 0px 7%;"><span style="width:100%"></span></div>
				</div>
			</div>

			<!-- Content -->
			<div id="wwcAmzAff-content">
				
				<h1 class="wwcAmzAff-section-headline">
					<?php 
					if( isset($this->module['synchronization']['in_dashboard']['icon']) ){
						echo '<img src="' . ( $this->module_folder . $this->module['synchronization']['in_dashboard']['icon'] ) . '" class="wwcAmzAff-headline-icon">';
					}
					?>
					<?php echo $this->module['synchronization']['menu']['title'];?>
					<span class="wwcAmzAff-section-info"><?php echo $this->module['synchronization']['description'];?></span>
					<?php
					$has_help = isset($this->module['synchronization']['help']) ? true : false;
					if( $has_help === true ){
						
						$help_type = isset($this->module['synchronization']['help']['type']) && $this->module['synchronization']['help']['type'] ? 'remote' : 'local';
						if( $help_type == 'remote' ){
							echo '<a href="#load_docs" class="wwcAmzAff-show-docs" data-helptype="' . ( $help_type ) . '" data-url="' . ( $this->module['synchronization']['help']['url2'] ) . '">HELP</a>';
						} 
					}
					echo '<a href="#load_docs" class="wwcAmzAff-show-feedback" data-helptype="' . ( 'remote' ) . '" data-url="' . ( $this->the_plugin->feedback_url ) . '" data-operation="feedback">Feedback</a>';
					?>
				</h1>

				<!-- Container -->
				<div class="wwcAmzAff-container clearfix">

					<!-- Main Content Wrapper -->
					<div id="wwcAmzAff-content-wrap" class="clearfix" style="padding-top: 20px;">

						<!-- Content Area -->
						<div id="wwcAmzAff-content-area">
							<div class="wwcAmzAff-grid_4">
	                        	<div class="wwcAmzAff-panel">
	                        		<div class="wwcAmzAff-panel-header">
										<span class="wwcAmzAff-panel-title">
											<?php _e('Synchronization logs', $this->the_plugin->localizationName);?>
										</span>
									</div>
									<div class="wwcAmzAff-panel-content">
										<form class="wwcAmzAff-form" action="#save_with_ajax">
											<div class="wwcAmzAff-form-row wwcAmzAff-table-ajax-list" id="wwcAmzAff-table-ajax-response">
											<?php
											wwcAmzAffAjaxListTable::getInstance( $this->the_plugin )
												->setup(array(
													'id' 				=> 'wwcAmzAffSyncMonitor',
													'show_header' 		=> true,
													'search_box' 		=> false,
													'items_per_page' 	=> '10',
													'post_statuses' 	=> array(
														'publish'   => __('Published', $this->the_plugin->localizationName)
													),
													'list_post_types'	=> array('product', 'product_variation'),
													'show_parent_products' => true,
													'columns'			=> array(

														'id'		=> array(
															'th'	=> __('ID', $this->the_plugin->localizationName),
															'td'	=> '%ID%',
															'width' => '40'
														),
														
														'thumb'		=> array(
															'th'	=> __('Thumb', $this->the_plugin->localizationName),
															'td'	=> '%thumb%',
															'align' => 'center',
															'width' => '50'
														),
														
														'asin'		=> array(
															'th'	=> __('ASIN', $this->the_plugin->localizationName),
															'td'	=> '%asin%',
															'align' => 'center',
															'width' => '70'
														),

														'title'		=> array(
															'th'	=> __('Title', $this->the_plugin->localizationName),
															'td'	=> '%title%',
															'align' => 'left'
														),
														
														'price'		=> array(
															'th'	=> __('Price', $this->the_plugin->localizationName),
															'td'	=> '%price%',
															'align' => 'left',
															'width' => '160'
														),
														
														'last_sync_date' => array(
															'th'	=> __('Last Sync Date', $this->the_plugin->localizationName),
															'td'	=> '%last_sync_date%',
															'width' => '160'
														),
														
														'date'		=> array(
															'th'	=> __('Date Added', $this->the_plugin->localizationName),
															'td'	=> '%date%',
															'width' => '120'
														),
														
														'parent_id'		=> array(
															'th'	=> __('Parent ID', $this->the_plugin->localizationName),
															'td'	=> '%parent_id%',
															'width' => '60'
														),

														'optimize_btn' => array(
															'th'	=> __('Action', $this->the_plugin->localizationName),
															'td'	=> '%button%',
															'option' => array(
																'value' => __('Sync Now', $this->the_plugin->localizationName),
																'action' => 'sync_now'
															),
															'width' => '80'
														),
													)
												))
												->print_html();
								            ?>
								            </div>
							            </form>
				            		</div>
								</div>
							</div>
							<div class="clear"></div>
						</div>
					</div>
				</div>
			</div>
		</div>

<?php
		}

		/*
		* ajax_request, method
		* --------------------
		*
		* this will create requesto to 404 table
		*/
		public function ajax_request()
		{
			global $wpdb;
			$request = array(
				'id' 			=> isset($_REQUEST['id']) ? (int)$_REQUEST['id'] : 0
			);
			
			$asin = get_post_meta($request['id'], '_amzASIN', true);
			
			$sync = new wwcAmazonSyncronize( $this->the_plugin );
			$sync->updateTheProduct( $asin );
		}
    }
}

// Initialize the wwcAmzAffTailSyncMonitor class
//$wwcAmzAffTailSyncMonitor = new wwcAmzAffTailSyncMonitor($this->cfg, $module);
$wwcAmzAffTailSyncMonitor = wwcAmzAffTailSyncMonitor::getInstance();